part of mila.test;

class DummyModel extends Model {
  @field String aha;
}

class Podo extends Model {
  @field String name;
  @field int age;
  @field bool thisIsNull;
}

class WithOneToOne extends Podo {
  @field DummyModel dummyModel;
}

class WithOneToMany extends Podo {
  @field List<DummyModel> dummyModels;
}

class KindaNested extends Podo {
  @field List<Map<String, List<DummyModel>>> nested;
}

class WithGetter extends Podo {
  @field String get hello => 'hello my name is $name and I\'m $age years old';
}

void modelAndModelFactoryTest() {
  group('Model and ModelFactory test', () {

    group('factory', () {

      test('creating simple Model', () {
        var factory = new ModelFactory(Podo),
            data = {'name' : 'faris',
                    'age' : 22},
            podo = factory.create(data);

        expect(podo.name, equals('faris'));
        expect(podo.age, equals(22));
        expect(podo.thisIsNull, isNull);
      });

      test('creating model with one to one relationship', () {
        var factory = new ModelFactory(WithOneToOne),
            firstData = {'name' : 'faris',
                         'age' : 22},
            secondData = {'name' : 'faris',
                          'age' : 22,
                          'dummyModel' : {'aha' : 'testtest'}},
            firstPodo = factory.create(firstData),
            secondPodo = factory.create(secondData);

        expect(firstPodo.name, equals('faris'));
        expect(firstPodo.age, equals(22));
        expect(firstPodo.thisIsNull, isNull);
        expect(firstPodo.dummyModel, isNull);

        expect(secondPodo.name, equals('faris'));
        expect(secondPodo.age, equals(22));
        expect(secondPodo.thisIsNull, isNull);
        expect(secondPodo.dummyModel.aha, equals('testtest'));
      });

      test('creating model with one to many relationship', () {
        var factory = new ModelFactory(WithOneToMany),
            firstData = {'name' : 'faris'},
            secondData = {'name' : 'faris',
                          'age' : 22,
                          'dummyModels' : [{'aha' : 'ehe'},
                                           {'aha' : 'ihi'}]},
            firstPodo = factory.create(firstData),
            secondPodo = factory.create(secondData);

        expect(firstPodo.name, equals('faris'));
        expect(firstPodo.age, isNull);
        expect(firstPodo.thisIsNull, isNull);
        expect(firstPodo.dummyModels, isNull);

        expect(secondPodo.name, equals('faris'));
        expect(secondPodo.age, equals(22));
        expect(secondPodo.dummyModels.first.aha, equals('ehe'));
        expect(secondPodo.dummyModels.last.aha, equals('ihi'));
      });

      test('creating model with nested type', () {
        var factory = new ModelFactory(KindaNested),
            firstData = {'name' : 'faris',
                         'age' : 22,
                         'nested' : [{'first' : [{'aha' : 'ehe'},
                                                 {'aha' : 'ehehe'}]},
                                     {'first' : [{'aha' : 'fhfhfh'},
                                                  {'aha' : '2323'}]}]},
            secondData = {'name' : 'faris',
                          'age' : 22},
            firstPodo = factory.create(firstData),
            secondPodo = factory.create(secondData);

        expect(firstPodo.name, equals('faris'));
        expect(firstPodo.age, equals(22));
        expect(firstPodo.nested.first['first'].first.aha, equals('ehe'));
        expect(firstPodo.nested.first['first'].last.aha, equals('ehehe'));
        expect(firstPodo.nested.last['first'].first.aha, equals('fhfhfh'));
        expect(firstPodo.nested.last['first'].last.aha, equals('2323'));

        expect(secondPodo.name, equals('faris'));
        expect(secondPodo.age, equals(22));
      });

    });

    group('to map', () {

      test('all null', () {
        var podo = new Podo(),
            asMap = podo.toMap();

        expect(asMap.containsKey('name'), isTrue);
        expect(asMap.containsKey('age'), isTrue);
        expect(asMap.containsKey('thisIsNull'), isTrue);
        expect(asMap['name'], isNull);
        expect(asMap['age'], isNull);
        expect(asMap['thisIsNull'], isNull);
      });

      test('simple model', () {
        var podo = new Podo()
              ..name = 'faris'
              ..age = 22,
            podoAsMap = podo.toMap();

        expect(podoAsMap['name'], equals('faris'));
        expect(podoAsMap['age'], equals(22));
        expect(podoAsMap.containsKey('thisIsNull'), isTrue);
        expect(podoAsMap['thisIsNull'], isNull);
      });

      test('with one to one relationship', () {
        var oneToOne = new WithOneToOne()
              ..name = 'faris'
              ..age = 22
              ..dummyModel = (new DummyModel()..aha = 'ehe'),
            asMap = oneToOne.toMap();

        expect(asMap['name'], equals('faris'));
        expect(asMap['age'], equals(22));
        expect(asMap['dummyModel']['aha'], equals('ehe'));
      });

      test('with one to many relationship', () {
        var oneToMany = new WithOneToMany()
              ..name = 'faris'
              ..age = 22
              ..dummyModels = [new DummyModel()..aha = 'ehe',
                               new DummyModel()..aha = 'oho'],
            asMap = oneToMany.toMap();

        expect(asMap['name'], equals('faris'));
        expect(asMap['age'], equals(22));
        expect(asMap['dummyModels'], isList);
        expect(asMap['dummyModels'].first['aha'], equals('ehe'));
        expect(asMap['dummyModels'].last['aha'], equals('oho'));
      });

      test('nested', () {
        var nested = new KindaNested()
              ..name = 'faris'
              ..age = 22
              ..nested = [
                {'foo' : [new DummyModel()..aha = 'ahafirst',
                          new DummyModel()..aha = 'ahasecond']},
                {'bar' : [new DummyModel()..aha = 'ehefirst',
                          new DummyModel()..aha = 'ehesecond']}
              ],
           asMap = nested.toMap();

        expect(asMap['name'], equals('faris'));
        expect(asMap['age'], equals(22));

        var first = asMap['nested'].first,
            second = asMap['nested'].last;

        expect(first['foo'].first['aha'], equals('ahafirst'));
        expect(first['foo'].last['aha'], equals('ahasecond'));
        expect(second['bar'].first['aha'], equals('ehefirst'));
        expect(second['bar'].last['aha'], equals('ehesecond'));
      });

    });

    test('with getter', () {
      var withGetter = new WithGetter()
        ..name = 'faris'
        ..age = 22;

      expect(withGetter.name, equals('faris'));
      expect(withGetter.age, equals(22));
      expect(withGetter.hello, equals('hello my name is faris and I\'m 22 years old'));
    });

  });
}